@@ -128,7 +128,7 @@ static Boolean is_member(const char *  item, const char * list[])
 static void initialize_terminal(void)
 {
 	const char *color_terms[] = { "linux", "xterm", "xterm-color", "xterm-256color",
-		"Eterm", "rxvt", "rxvt-unicode", 0};
+		"Eterm", "rxvt", "rxvt-unicode", "cygwin", 0};
 	const char *no_ansi_cursor_terms[] = {"dumb", "emacs", 0};
 	const char *term = getenv("TERM");
 	if (term == 0)
@@ -441,7 +441,14 @@ int main(int argc, char *argv[])
 		exit(EX_OK);
 	}
 
+#ifdef WIN32
+	if (Options.verbose)
+	{
+		printf("%sPress shift key to quit%s\n", magenta, color_end);
+	}
+#else
 	initialize_signal_handlers();
+#endif
 
 	rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
 	test_rv("SCardEstablishContext", rv, hContext);
@@ -541,6 +548,10 @@ get_readers:
 			{
 				rv = SCardGetStatusChange(hContext, TIMEOUT, rgReaderStates, 1);
 				spin_update();
+#ifdef WIN32
+				if (GetKeyState(VK_SHIFT) & 0x80)
+					spinning_interrupted = TRUE;
+#endif
 			}
 			while ((SCARD_E_TIMEOUT == rv) && !spinning_interrupted);
 			spin_suspend();
@@ -620,6 +631,10 @@ get_readers:
 		nbReaders++;
 	}
 
+#ifdef WIN32
+	int oldNbReaders;
+	int oldNbReaders_init = FALSE;
+#endif
 	spin_start();
 
 	/* Wait endlessly for all events in the list of readers
@@ -632,10 +647,15 @@ get_readers:
 
 		if (pnp)
 		{
-#ifdef WIN32
 			/* check if the number of readers has changed */
+#ifdef WIN32
 			LONG newNbReaders = rgReaderStates_t[nbReaders-1].dwEventState >> 16;
-			if (newNbReaders + 1 != nbReaders)
+			if (! oldNbReaders_init)
+			{
+				oldNbReaders = newNbReaders;
+				oldNbReaders_init = TRUE;
+			}
+			if (newNbReaders != oldNbReaders)
 #else
 			if (rgReaderStates_t[nbReaders-1].dwEventState &
 					SCARD_STATE_CHANGED)
@@ -801,6 +821,11 @@ get_readers:
 			nbReaders);
 
 		spin_update();
+
+#ifdef WIN32
+	if (GetKeyState(VK_SHIFT) & 0x80)
+		spinning_interrupted = TRUE;
+#endif
 	} /* while */
 
 	spin_suspend();
